"""All authentication exceptions"""

from .app import BaseAppError


class AuthFailure(BaseAppError):
    """Authentication Failed"""
    message = 'Authentication or authorization failure'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class APIAuthFailure(AuthFailure):
    """API Token Authentication Failed"""
    message = 'API key is invalid for URL'

    def __init__(self, input_var, *args, **kwargs):
        super().__init__(input_var=input_var, *args, **kwargs)


class APIAuthNotAuthorized(AuthFailure):
    """API Token Auth Not Authorized"""
    message = 'API token auth is not authorized, or token auth is not supported for the given URL'

    def __init__(self, input_var, *args, **kwargs):
        super().__init__(input_var=input_var, *args, **kwargs)


class PasswordAuthFailure(AuthFailure):
    """Password Authentication Failed"""
    message = 'Password authentication or authorization failure'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)



from .base import LhBaseException
from .app import BaseAppError


class BaseValidationError(LhBaseException):
    """Base exception for format errors for LogicHub resource objects"""
    message = "Validation failure"

    def __init__(self, message=None, *args, **kwargs):
        self.message = message or "Input validation failed"
        super().__init__(self.message, *args, **kwargs)


class InputValidationError(BaseValidationError, ValueError):
    """Invalid format for input"""
    message = 'Invalid input'

    def __init__(self, input_var, action_description=None, *args, **kwargs):
        if action_description:
            self.message += f" for {action_description}"
            self.action_description = action_description
        super().__init__(input_var=input_var, *args, **kwargs)


class ResponseValidationError(BaseValidationError, BaseAppError):
    """Unexpected schema for LogicHub API response"""
    message = 'API response received in unexpected format/schema'

    def __init__(self, input_var, *args, **kwargs):
        super().__init__(input_var=input_var, *args, **kwargs)


class VersionMinimumNotMet(BaseValidationError):
    """Minimum version not met"""
    message = 'Requested action ({}) requires a minimum LogicHub version of {}.'

    def __init__(self, min_version, feature_label, *args, **kwargs):
        self.message = self.message.format(feature_label, min_version)
        self.id = feature_label
        super().__init__(self.message, *args, **kwargs)



import logging


# Declaring these things all the time results in logging config changes that
# cannot easily be undone, so only perform them when invoked
def prep_generic_logger(level=None):
    # Only run once, though.
    if hasattr(prep_generic_logger, "done"):
        return
    level = level.upper().strip() if level else None
    logging.basicConfig(level=level)
    logger = logging.getLogger()
    if not level:
        logger.disabled = True
    prep_generic_logger.done = True



import datetime

DEFAULT_TIME_FORMAT = "%Y-%m-%d %H:%M:%S"


def epoch_time_to_str(time_sec, time_format=None):
    time_format = time_format if time_format else DEFAULT_TIME_FORMAT
    dt = datetime.datetime.fromtimestamp(time_sec)
    return dt.strftime(time_format)



from .base import LhBaseException


# ToDo Rework exceptions just like I did for lhub_cli
class BaseAppError(LhBaseException):
    """Base exception for failures from interacting with LogicHub itself, including custom HTTP errors"""
    message = "LogicHub returned an error"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class AlertQueryValidationError(BaseAppError):
    """Invalid format for alert queries"""
    message = 'Alert query is invalid'

    def __init__(self, input_var, *args, **kwargs):
        super().__init__(input_var=input_var, *args, **kwargs)


class CaseQueryValidationError(BaseAppError):
    """Invalid format for case queries"""
    message = 'Case query is invalid'

    def __init__(self, input_var, *args, **kwargs):
        super().__init__(input_var=input_var, *args, **kwargs)


class BatchNotFound(BaseAppError):
    """Batch/Batch ID not found"""
    message = 'Batch not found'

    def __init__(self, input_var, *args, **kwargs):
        super().__init__(input_var=input_var, *args, **kwargs)


class NotebookNotFound(BaseAppError):
    """Batch ID not found"""
    message = 'Notebook not found'

    def __init__(self, input_var, *args, **kwargs):
        super().__init__(input_var=input_var, *args, **kwargs)


class RuleSetNotFound(BaseAppError):
    """Rule set not found for the given search criteria"""
    message = 'Rule set not found'

    def __init__(self, input_var, *args, **kwargs):
        super().__init__(input_var=input_var, *args, **kwargs)


class StreamNotFound(BaseAppError):
    """Stream ID not found"""
    message = 'Stream not found'

    def __init__(self, input_var, *args, **kwargs):
        super().__init__(input_var=input_var, *args, **kwargs)


class CannotStartStreamException(BaseAppError):
    """Stream could not be started"""
    message = 'Stream could not be started'

    def __init__(self, input_var, *args, **kwargs):
        super().__init__(input_var=input_var, *args, **kwargs)


class UnexpectedOutput(BaseAppError):
    """Unexpected Output"""
    message = 'Unexpected output returned'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class UserGroupNotFound(BaseAppError):
    """User group not found"""
    message = 'User group not found'

    def __init__(self, input_var, *args, **kwargs):
        super().__init__(input_var=input_var, *args, **kwargs)


class UserGroupAlreadyExists(BaseAppError):
    """User group already exists"""
    message = 'User group already exists'

    def __init__(self, input_var, *args, **kwargs):
        super().__init__(input_var=input_var, *args, **kwargs)


class UserNotFound(BaseAppError):
    """User not found"""
    message = 'User not found'

    def __init__(self, user, *args, **kwargs):
        self.user = user
        super().__init__(input_var=user, *args, **kwargs)


class UserAlreadyExists(BaseAppError):
    """User already exists"""
    message = 'User already exists'

    def __init__(self, user, *args, **kwargs):
        self.user = user
        super().__init__(input_var=user, *args, **kwargs)
